%% Curve fit for first principle characterstic
% MODSIM, Egeland and Gravdahl 2002
% Turbocharger GT2252 data
% 08.02.2017, N.T.
% REV: 10.02, 14/3,
clear all; close all; clc;
% Notes:
%       - Conversion from rotational speed (omega) to rev. per. min in f
%       - beta1b, 2b are measured from the actual impeller

%% Load data 
%
wgc_mtp_params; % call struct for gt2252 and gas properties
%
%load('datafile0602.mat'); % load all the map data
load('datafile0103.mat'); % load data, rev 01.03
% structure of datafile:
%            
%          sp17 - vector with speed line data, bolk length  = n
%               [0w 0psi
%               [1w 1psi...
%               [7iw 7i psi] % last two rows is tuned data 
%          spd - vector with speed line data, zero data, bolk length = m
%               [0w 0psi
%               [1w 1psi...
%               [7iw 7i psi] % last two rows is tuned data
%          spdf - ...., zero and negative data, bolk length = f
%               [0w 0psi
%               [1w 1psi...
%               [6iw 6i psi] % last two rows is tuned data
%               [7iw 7i psi %
%          map815 - vector with map data, surge choke efficiency, length nm
%               [08w 08psi, surge ( small line)
%               [09w 09psi... choke (65 perc.)
%               [10w  10psi   surge (66 line)
%               [15w 15psi]      70, 75, 76, 77 and 78 (15)%

%% Establishing data for the function
% remark: (1/2-17) The data is the simplified, tuned as for the strict 
%                   empirical cubic poly  found.
wscale = 1; % Scale dataset massflow 10^2 (in dim. 1.0)
nscale = 1; % Scale dataset 10^-5 (in dim. 1.0)
step = 1; % step into data vector, 1 - exclude neg. data point

xdata = [spdf(1+step:f,1); spdf(f+1+step:f*2,1); spdf(f*2+1+step:f*3,1);...
         spdf(f*3+1+step:f*4,1); spdf(f*4+1+step:f*5,1); ...
         spdf(f*7+1+step:f*8,1); spdf(f*8+1+step:f*9,1)]*wscale; 

ydata = [spdf(1+step:f,2); spdf(f+1+step:f*2,2); spdf(f*2+1+step:f*3,2);...
         spdf(f*3+1+step:f*4,2); spdf(f*4+1+step:f*5,2); ...
         spdf(f*7+1+step:f*8,2); spdf(f*8+1+step:f*9,2)]; 
     
% Note: change to last elements with spdf(f*5+1:f*6,1), spdf(f*6+1:f*7,1)
%           to use un-tuned data

% Create 7 column rpm matrix for each speed line.
Nmat = @(length)[linspace(75,75,length); linspace(95,95,length);...
        linspace(115,115,length);   linspace(135,135,length);...
        linspace(155,155,length);   linspace(175,175,length); ...
        linspace(195,195,length)]';
% Nmat(:,7) gives the 195 column
Nd = Nmat(f-step); % Matrix with f rows

ndata = [Nd(:,1); Nd(:,2); Nd(:,3);...
            Nd(:,4); Nd(:,5); Nd(:,6); Nd(:,7)]*10^3*nscale; 

%% Final dataset to lsqcurvefit
xndata = [xdata ndata]; % [mass flow       rpm1 ]
                        % [ mass flow 2..   rpm2..]
yndata = ydata; % [ psi1 psi2 ... psin]                       
%% Curve fit, with bounds

% Magnitude of x data .. -0.05 0.2
% Magnitude of n data .. 75 - 195 *10^3

% Notes on constrains on coefficiants:
%       - cot(beta), from -1 to 1
%       - assume that sin(beta1b) > 0 then  0 < kf < limit

% Lower and upper bound limit for coefficients
% sigma*r2^2   cot(beta2b)/   r1^2/      alpha    kf     1/     kappa/
%              rho1*A1*r2      2                       cp*T01  kappa-1  
lb = [0,           0,             0,      0,       0,     0,     0    ]; 
ub = [1,           10e3,          1,     5e5,     1e6,   1e-1,   5    ]; 
%lb = [];ub = []; % no limits
options = optimoptions('lsqcurvefit','FunctionTolerance',1e-06,...
    'Algorithm','trust-region-reflective'); %Iteration step tolerance
                                        %Solver method, option 2:
                                        %'trust-region-reflective'
                                        
x0 = [5.4e-4, 8.2e3, 0.0002,  3.7e4,  2.5e5, 3.5e-6, 3.5 ]; % initial guess

[xn,resnorm,residual,exitflag,output] = lsqcurvefit(@psiC,...
               x0,xndata,yndata,lb,ub,options);
           
x0 = round(x0,2,'significant'); % round to two sign. digits
xn = round(xn,2,'significant');

coefficiants = {'sigmar2^2';'cot(beta2b)/rho1A1r2';'r1^2/2';'alpha';...
        'kf';'1/cpT01';'kappa/kappa-1'};
T = table(x0',xn'...
    ,'VariableNames',{'X0','XN'},'RowNames',coefficiants)
name = {'Counter';'Residual';'ExitFlag'};
T2 = table([output.funcCount resnorm exitflag]','VariableNames',...
    {'value'},'RowNames',name)
% 
% %% Curve fit data, with known parameters (process and geometric)
% wgc_mtp_params; % load params for process and turbo gt2252
%                 % This result will not be used. Poor fit.
% % sigmar22 = (1 - 2/gt2252.nblades)*gt2252.r2^2; % x1 - sigma * r2^2
% % phicotbeta2b = cot(gt2252.beta2b)/(process.rho1*gt2252.A1*gt2252.r2); 
% % % x2 - cot(beta2b)/rho1*A1*r2
% % r122 = 0.5*gt2252.r1^2; % x3 - 0.5*r1^2
% % cpT = 1/(process.T01*process.cp); % x6 - 1/cp*T01
% % kpp = process.kappa/(process.kappa - 1); % x7 - kappa/kappa -1
% % params = [sigmar22 phicotbeta2b r122 cpT kpp];
% % 
% % % Lower and upper bound limit for coefficients
% % %     alpha    kf    
% % lb = [0,        0 ]; 
% % ub = [5e7,     1e6 ];
% % x02 = [  3e4,  1e5 ];
% % 
% % [xn2,resnorm2,residual2,exitflag2,output2] = ...
% %     lsqcurvefit(@(x,xdata)psiCNested(x,xdata,params),...
% %                x02,xndata,yndata,lb,ub,options);
% % 
% % x02 = round(x02,2,'significant'); % round to two sign. digits
% % xn2 = round(xn2,2,'significant');
% % 
% % T3 = table(x02',xn2'...
% %     ,'VariableNames',{'X02','XN2'},'RowNames',{'Alpha';'Kf'})
% % name = {'Counter';'Residual';'ExitFlag'};
% % T4 = table([output2.funcCount resnorm2 exitflag2]','VariableNames',...
% %     {'value'},'RowNames',name)
% 
% % Lower and upper bound limit for coefficients
% %       Ch*Lm/2*Dm   beta1b   beta2b  
% lb = [      0,           0,     0   ]; 
% ub = [      1,           1,     1.5  ]; 
% %lb = [];ub = []; % no limits
% options = optimoptions('lsqcurvefit','FunctionTolerance',1e-06,...
%     'Algorithm','trust-region-reflective'); %Iteration step tolerance
%                                         %Solver method, option 2:
%                                         %'trust-region-reflective'
% pparams = process;
% cparams = gt2252;
% 
% x02 = [ 0.4   0.38   0.95 ]; % initial guess
% 
% [xn2,resnorm2,residual2,exitflag2,output2] = ...
%    lsqcurvefit(@(x,xdata)psiCParams(x,xdata,pparams,gt2252),...
%               x02,xndata,yndata,lb,ub,options);
% 
% x02 = round(x02,2,'significant'); % round to two sign. digits
% xn2 = round(xn2,2,'significant');
% 
% T3 = table(x02',xn2'...
%     ,'VariableNames',{'X02','XN2'},'RowNames',...
%     {'ChLm/2Dm';'beta1b';'beta2b'})
% name = {'Counter';'Residual';'ExitFlag'};
% T4 = table([output2.funcCount resnorm2 exitflag2]','VariableNames',...
%     {'value'},'RowNames',name)
