%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Simulation script for Centrifugal Compressor model
%
% Modeling and Control of Wet Gas Compressors
%
% Egeland 2002 - Centrifugal model for control av wet gas extension.
%
% Rev   01 - 22/3-17, N.T.
%  -    02 - 18/4-17, N.T.'
%  -    03 - 19/4-17, N.T.
%  -    04 - 24/4-17, N.T.- added deadtime on Controller output
%                           and noise on input (p01) and measurement noise
%                               on pp
%  -    05 - 4/5-17, N.T.
%
% Info: The model in Simulink is with both manual and PID control with
% tracking of output from the "Motor"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all; % Clear workspace variables, close figure/windows

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Load content from files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load('init_files/datafile0103.mat'); % Compressor map data for GT2252
load('init_files/datafile0103coeffs.mat'); % Empirical coefficient data
load('init_files/psiParams.mat'); % First principle coefficient data

xmatEmp = [c0 c1 c2 c3]; % Set empirical coeff. for empPsi
xmat = [x1 x2 x3 x4 x5 x6 x7]; % Set first-principle coeff. for psiC

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Laboratory parameters, WGC- MTP
% Modeling and Control of Wet Gas Compressors
%
% Rev   01 - 19/3-17, N.T.
%
% --- Process parameters
cp = 1004.6; % ~ Air at 293.15K, from Boosting lab calc. 20C J/kg.K
cv = 717.4; % ~ From boosting lab calc
kappa = cp/cv; % ratio of specific heats
T01 = 293.15; % Kelvin, compressor inlet temp at 20C standard cond.
rho1 = 1.205; % density of air at compressor inlet
a0 = 343; % m/s, sonic speed at ambient conditions (1 atm, dry gas)
p0 = 1.01325e5; %Pa, 1 atm

% --- Compressor params.
r1 = (40.2e-3)/2; % radii in m
r2 = (52.0e-3)/2;
A1 = ((60*10^-3)/2)^2*pi; %inlet area
A2 = ((50*10^-3)/2)^2*pi; %outlet area
beta1b = 21.4*(2*pi/360); %blade inducer/entry angle, rad
beta2b = 54.1*(2*pi/360); %blade exit angle, rad
nblades = 12; % compressor impeller blades
trim_c = ((0.0402^2)/(0.052^2))*100; %inducer-to exducer ratio
Vp = 2.45e-4 + 0.0023; %m^3, Compressor plenum volume for set-up (fig ref.)
Ac = (A1 + A2)/2; % Compressor flow through area, averaged
Lc = 0.5; % 50 cm Equvialent length from inlet to discharge
At = (0.051/2)^2*pi; % Opening of throttle valve in WGC lab.
Lt = 0.05; % length of valve section ~ 5 cm
% - end lab params

f = 0.0188; % friction factor, Re approx 80000 turb. flow
Ch = 4*f; % Surface friction coefficient (Gravdahl 1998)
lm = 0.04; % Mean channel length impeller
Dm = 3.75e-3; % Mean channel diameter impeller
allpha = cot(beta1b)/(rho1*A1*r1); % Incidence loss coefficient
sigma = 1 - 2/nblades; % Stanitz's slip factor

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Compresor Initial conditions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pp_init = 1.2e5; %Pa, plenum pressure at t=0
w_init = 0.12; %kg/s, compressor mass flow at t=0 (ideal eff. area)
%omega_init = 135000*2*pi/60; % Starting speed (rad/s) of compressor
p01 = p0; %Pa, compressor inlet suction pressure at inducer (atm)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Simulation section
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rpm_max = 195000;
rpm_min = 0;
o_max = rpm_max*2*pi/60;
o_min = rpm_min*2*pi/60;

% ----- Controller settings, c. mdl.
u_man = 135000; % rpm, Actuating signal for no controller setup
man_aut = 0; % man = 0, aut = 1
t_manaut = 0; % step time to initiate controllers
u_max = rpm_max; % sat. threshold for compressor drive
u_min = rpm_min; % -

% ----- Actuator settings
wt_max = 0.25; % sat. threshold for throttle
wt_min = 0; % 

%  GS CO2 - Gain scheduling controller params
gs_on = 1; % GS on = 1, off = 0;
t_gs_on = 0; % step time to initiate GS controller
yf_init = w_init; % filtered gs variable at t=0, equal to comp. mass flow
Tf = 0.01; % Smooth transition between Kp, Ti for GS control
           % first order filter
gs_filter = 0; % 1 - 1st order filter on GS var. with Tf filter cnst.
gainCnst = 0; % 0 - GS on Kp, 1 - GS on only Ti
gs_kc = 2.4521; %0.46; % constant gs proportional gain
gainInc = 0; % boolean 0 - increase Kp gain and Ti integral time
gs_gain = [gainCnst gs_kc gainInc 1 1]; % used by GS fcn 

% ---- CO1, single, parameters.
tc = 0.01; % closed loop response time, tuning param., SIMC- tuning rules
K = 2.5; tau1 = 0.024; dth = 0.0005;
Kp = (1/K)*(tau1/(tc + dth)); % Proportional gain
Ti = min(tau1,4*(tc+dth)); % Integral time, minimum of the two params
Td = 0; % Derivative time
%Kp = 0.4762; Ti = 0.021;  % skogestad param. tuning01
%Kp = 0.4; Ti = 0.024; Td = 0; % Skogestad param. Tuning02  tc = tau1
%Kp = 9.6; Ti = 0.004; Td = 0; % Skogestad param. Tuning w/ time del.
%Kp = 13.49; Ti = 0.003; Td = 0; % ?strom Hagglund 2002 PI KLT tuning
%Kp = 6.72; Ti = 0.0035; Td = 0; % ?strom Hagglund 2002 PI L/T -> 0

track =1; % 0 - None, 1 - tracking/back-calc (antiwindup)
Tt = 1; % Track/back-calc. dyn. for PID
df = 0; % filter coeff for derivative gain

% ---- Step settings
t_rpm_step = 0; % Step for speed (centrifugal model w/o drive only)
rpm_step_i = 0; % Speed at t_step = 0,
rpm_step_f = 0; % final added speed val.

pp_sp = 1.8e5; % SP for plenum pressure
t_sp_step = 0; % Time step for SP change
pp_sp_step_i = 0; % added to SP at t_step = 0
pp_sp_step_f = 0e5;% added to SP at t_step

kt_init = 0.8; % Throttle Control, Throttle opening at start
t_kt_step = 0.1; % time to init. throttle step
kt_step_i = 0; % initial subtracted opening
kt_step_f = 0.6; % reduction in valve opening (kt_init - kt_step_f)
% wt_gs = 0.09; % Manual adjustment to throttle flow
% GS var: 0.0900    0.1120    0.1340    0.1560    0.1780    0.2000

% ---- Compressor charactestic settings
ch_char = 0; % 0- first principle, 1 - empirical

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Delay, disturbance and simulation time
% - sin(t) = p_in_amp*Sin(p_in_f*t+phase) + Bias, bias = p01 cnst.
p_in_amp = 0.001e5; % amplitude of sinusoidal inlet pressure noise
p_in_f = 400; % frequency of sinusoidal inlet pressure noise rad/s
w_out_amp = 0; % amp 
w_out_f = 0;

vf_min = -pp_sp*0.001; % added noise +-0.1%
vf_max = -vf_min; %

Ts=0.0001; % simulation timestep, change if using fixed step
tau_cd = 10*Ts; % reasonable delay time for actuating signal (u-cntrl)
Ttd = Tf; % Shaft dynamics, turbine delay in control
uf0 = u_man; % Shaft dynamics, time constant
t_start=0; t_stop=0.2; %sek, ms... change this to prolong sim
timespan = [t_start,t_stop];

% Wet gas parameter changes
rhog = 1.205; %gas density (air)
rhol =998.21; %liquid density (water)
cpg=1004.6; cvg = 717.4; cpw = 4182; cvw = cpw; 

sim_wgc = 0; % simulate with "wet gas behaviour"
t_gvf_step = 0.1; % time to ramp down gvf for flow
gvf = [100 99.97 99.95 99.93 99.9 99]*1/100; %
gvf_cnst =gvf(1);%
gvf_max = gvf_cnst; % value used if sim_wgc = 0;
gmf_cnst = gvf_cnst*rhog/(gvf_cnst*rhog + (1 - gvf_cnst)*rhol);
gmf_max = gmf_cnst;
ramp_slope = -0.01; % slope of GVF ramp
gmf_ramp_slope = -5; % slope of GMF ramp
% gmf used for friction loss coeff.

dry_gas = [rhog, cpg]; % parameters for wet gas flow

sim_options = simset('solver','ode14x','fixedstep',Ts); % Solver properties
%sim_options = simset('solver','ode23s'); % ode23s stiff systems
sim('centrifugal_mdl',timespan,sim_options);

%  Solver - [ VariableStepDiscrete |
%             ode45 | ode23 | ode113 | ode15s | 
%             ode23s | ode23t | ode23tb |
%             FixedStepDiscrete |
%             ode8 | ode5 | ode4 | ode3 | ode2 | ode1 |
%             ode14x ]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Performance outcome of controller
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tsp = t_sp_step - t_start; % time from start to step
t_sp_num = tsp/Ts; % number of data steps
t_log_sp = tsp - 2*Ts; % start two timesteps before sp step
t_s_num = floor(t_log_sp/Ts); % start of IAE index log time for step SP

tpd = t_kt_step - t_start; % time from start to step of throttle
t_pd_num = tpd/Ts; % number of data steps
t_log_pd = tpd - 2*Ts; % start two timesteps before disturbance step
t_s_num_pd = floor(t_log_pd/Ts); % Start of IAE index log time for step dist.
t_f_num_pd = floor(t_stop/Ts); % stop of IAE index log time for dist.

% %Integral of Absolute value of control Error, set point and load dist.
if gs_on == 0 && man_aut==1 % calculate IAE for single PI controller
     %t_log = t_manaut - 2*Ts; % start time log
     %t_log = t_log/Ts; % num data steps
    if t_sp_step ~= 0
     IAEsp = Ts*(sum(abs(error...
        (t_s_num:t_s_num_pd-100)))) % -, set point change
    end
    if t_kt_step ~= 0
     IAEd = Ts*(sum(abs(error...
         (t_s_num_pd:t_f_num_pd))))  % -, disturbance change
    end
end
if gs_on == 1 % calculate IAE for gs controller
%    tgs_log = t_gs_on - 2*Ts; % start time log
%    tgs_log = tgs_log/Ts; % num data steps
%    IAEgs = Ts*(sum(abs(error(tgs_log:t_stop/Ts)))) % IAE for gs
    if t_sp_step ~= 0
    IAE_gs_sp = Ts*(sum(abs(error...
        (t_s_num:t_s_num_pd-100)))) % -, set point change
    end
    if t_kt_step ~= 0
     IAE_gs_d = Ts*(sum(abs(error...
         (t_s_num_pd:t_f_num_pd))))  % -, disturbance change
    end
end
if sim_wgc == 1 % calculate IAE for wet gas compression
    t_log = t_gvf_step - 4*Ts; % start time log
    n_log = t_log/Ts; % num data steps
    n_log_stop = t_stop/Ts; %end time of IAE log
    if t_gvf_step ~= 0
    IAE_wgc = Ts*(sum(abs(error...
        (n_log:n_log_stop)))) 
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Plot results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
init_plot; % Initialze plot settings

dpnt = 'k-'; % data point symbol and color, subplot2
dfit = dpnt; % 
c = 0.01; % offset plot x-dir
d = -0.09; % offset plot y-dir
% 
% sysplot = figure; % system variables
% subplot(2,2,1);
% plot(t_sim, p01_sim,'Linewidth',0.5);
% xlabel('Time [s]');
% ylabel('p_{01} [Pa]');
% grid on
% %axis([-inf inf  min(p01_sim)*0.9  max(p01_sim)*1.1])
% 
% subplot(2,2,2);
% plot(t_sim, w_sim.data);
% xlabel('Time [s]');
% ylabel('w [kg/s]');
% grid on
% %axis([-inf inf  -inf  max(w.data)*1.3])
% 
% subplot(2,2,3);
% plot(t_sim, pp_m_sim);
% xlabel('Time [s]');
% ylabel('p_p [Pa]');
% grid on
% axis([-inf inf  min(pp_m_sim)*0.9  max(pp_m_sim)*1.1]);
% 
% subplot(2,2,4);
% plot(t_sim, u_sim);
% xlabel('Time [s]');
% ylabel('Shaft speed [rpm]');
% grid on
% axis([-inf inf   min(us_sim)*0.9 max(us_sim)*1.2]);
% 
compmapplot = figure; % Compressor map plot
% - plot map data
plot(x1,y1,dpnt,'linewidth',0.5); text(x1(end)+c,y1(end)+d,'N_{75}');
hold on;
plot(x2,y2,dpnt,'linewidth',0.5); text(x2(end)+c,y2(end)+d,'N_{95}');
plot(x3,y3,dpnt,'linewidth',0.5); text(x3(end)+c,y3(end)+d,'N_{115}');
plot(x4,y4,dpnt,'linewidth',0.5); text(x4(end)+c,y4(end)+d,'N_{135}');
plot(x5,y5,dpnt,'linewidth',0.5); text(x5(end)+c,y5(end)+d,'N_{155}');
plot(x6,y6,dpnt,'linewidth',0.5); text(x6(end)+c,y6(end)+d,'N_{175}');
plot(x7,y7,dpnt,'linewidth',0.5); text(x7(end)+c,y7(end)+d, 'N_{195}');
plot(x10,y10,'k-','linewidth',0.6);
plot(x9,y9,'k--','linewidth',0.6);
plot(map815(91:120,1),map815(91:120,2),'k:','LineWidth',0.5); %70ef
plot(map815(211:240,1),map815(211:240,2),'k:','LineWidth',0.5); %78ef
%-- end plot map data
%   -- 
plot(w_sim.data,psi_sim.data,'Linewidth',1); % actual resulting data from sim
plot(w_sim.data(1),psi_sim.data(1),'k*'); % plot starting point for comp.
plot(w_sim.data(end),psi_sim.data(end),'kx'); % end point
% ----------------
hold on;
axis([-0.05 0.3  0.8  4])
title('Compressor map, centrifugal mdl','Fontsize',10,'FontWeight','Normal');
xlabel 'w [kg/s]'
ylabel '\Psi_c [-]'
grid on
% 
% Display control settings and behavior
cntrlplot = figure; % figure 2 controller plot
ax1 = subplot(3,1,1);
plot(t_sim,w_sim.data,t_sim,wt_sim.data,'--');
xlabel('Time [s]');
ylabel('w [kg/s]');
%title(['Throttle step at t=',num2str(t_kt_step),' | k_t from ',...
%    num2str(kt_init),' to ', num2str(kt_init-kt_step_f)],...
%    'Fontsize',10,'FontWeight','Normal');
grid on
legend('w','w_t');
axis([-inf inf  min(w_sim.data)*0.9   max(w_sim.data)*1.1]);

ax2 = subplot(3,1,2);
yyaxis left
plot(t_sim, pp_m_sim,'Linewidth',0.5);
hold(ax2,'on');
plot(t_sim, pp_sp_sim);
hold(ax2,'off');
yyaxis right
plot(t_sim,r_gvf_sim);
ylim([gvf_cnst*0.999 1.001])
xlabel('Time [s]');
yyaxis right
ylabel('GVF');
yyaxis left
ylabel('p_p [Pa]');
grid on
axis([t_start t_stop  min(pp_m_sim)*0.9  max(pp_m_sim)*1.1])

ax3 = subplot(3,1,3);
plot(t_sim, u_sim,t_sim, u_pi,'--');
xlabel('Time [s]');
ylabel('Shaft speed [rpm]');
legend('u_{shaft}','u_{pi}');
grid on
axis([-inf inf  min(u_sim)*0.9  max(u_sim)*1.1])

if man_aut == 1 && gs_on ==0
uplot = figure; % figure 4 controller plot
subplot(3,1,1);
plot(t_sim, up_sim);
xlabel('Time [s]');
ylabel('u_p');
grid on

subplot(3,1,2);
plot(t_sim, ui_sim);
xlabel('Time [s]');
ylabel('u_i');
grid on

subplot(3,1,3);
plot(t_sim, ud_sim);
xlabel('Time [s]');
ylabel('u_d');
grid on
end
% 
% % plot the Kp, and Ti for GS
if gs_on == 1

ktplot = figure; % kp ti of gs controller plot
subplot(2,1,1);
plot(t_sim, kp_gs.data);
xlabel('Time [s]');
ylabel('GS - K_c');
grid on

subplot(2,1,2);
plot(t_sim, Ti_gs.data);
xlabel('Time [s]');
ylabel('GS - \tau_i');
grid on

uplot = figure; % up and ui for gs controller plot
subplot(2,1,1);
plot(t_sim, up_gs_sim);
xlabel('Time [s]');
ylabel('GS - up');
grid on

subplot(2,1,2);
plot(t_sim, ui_gs_sim);
xlabel('Time [s]');
ylabel('GS - ui');
grid on
end


wgcplot = figure; % figure 4 controller plot
subplot(3,1,1);
plot(t_sim, wgcparams_sim(:,1));
xlabel('Time [s]');
ylabel('\rho_h');
grid on

subplot(3,1,2);
plot(t_sim, wgcparams_sim(:,2));
xlabel('Time [s]');
ylabel('c_h');
grid on

subplot(3,1,3);
plot(t_sim, dhf_sim);
%plot(t_sim, wgcparams_sim(:,3));
xlabel('Time [s]');
ylabel('k_f');
grid on

%%%%%%%%%%%%%%%%%%%%%
% Print to file
%%%%%%%%%%%%%%%%%%%%
%print(sysplot,'fig1_c_mdl_sys-sparam2','-depsc2','-r300');
%print(compmapplot,'fig2_c_mdl_compmap-','-depsc2','-r300');
%print(cntrlplot,'fig3_yt_rt_u-sparam2','-depsc2','-r300');
%print(uplot,'fig4_cntrl-u','-depsc2','-r300');


