%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Simulation script for Greitzer compressor model
% Modeling and Control of Wet Gas Compressors
%
% Greitzer 1976 - Axial compressor model
%
% Rev   01 - 22/3-17, N.T.
%  -    02 - 18/4-17, N.T.'
%  -    03 - 19/4-17, N.T.
%
% Info: Passive behaviour of model with constant speed in rpm
%        surge -> throttle to 10 %
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all; clc; %Clear workspace variables, close figure/windows

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Load content from files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load('init_files/datafile0103.mat'); % Compressor map data for GT2252
load('init_files/datafile0103coeffs.mat'); % Empirical coefficient data
load('init_files/psiParams.mat'); % First principle coefficient data

empCoeff = [c0 c1 c2 c3]; % Set empirical coeff. for empPsi

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Laboratory parameters, WGC - MTP
% Modeling and Control of Wet Gas Compressors
%
% Rev   01 - 19/3-17, N.T.
%
cp = 1004.6; % ~ Air at 293.15K, from Boosting lab calc. 20C J/kg.K
cv = 717.4; % ~ From boosting lab calc
cp_w = 419; % ~ ", water cp
kappa = cp/cv; % ratio of specific heats
T01 = 20+273.15; % Kelvin, inlet temp at 20C standard cond.
rho1 = 1.205; % density of air at inlet
rhop = rho1; % density of gas in the plenum
a0 = 343; % m/s, sonic speed at ambient conditions (1 atm, dry gas)
p0 = 1.01325e5; %Pa, 1 atm

r1 = (40.2e-3)/2; % radii in m
r2 = (52.0e-3)/2;
A1 = ((60*10^-3)/2)^2*pi; %inlet area
A2 = ((50*10^-3)/2)^2*pi; %outlet area
beta1b = 21.4*(2*pi/360); %blade inducer/entry angle, rad
beta2b = 54.1*(2*pi/360); %blade exit angle, rad
nblades = 12; % compressor impeller blades
trim_c = ((0.0402^2)/(0.052^2))*100; %inducer-to exducer ratio
Vp = 2.45e-4 + 0.0023; %m^3, Compressor plenum volume for set-up (fig ref.)
Ac = (A1 + A2)/2; % Compressor flow through area, averaged
Lc = 0.5; % 50 cm Equvialent length from inlet to discharge
At = (0.051/2)^2*pi; % Opening of throttle valve in WGC lab.
Lt = 0.05; % length of valve section ~ 5 cm

% - end lab params
Ns = 0.1; % Rotor rev. Stall cell growth (Greitzer 1976 article part II)
Rs = (r1+r2)/2; % Rotor mean radius
% Note: The valve dynamics can be neglected, due to small dimension on
%        the valve section

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Simulation section
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - uncomment sim cmd to use spec. mdl.
rpm_max = 195000;
rpm_min = 75000;

%%%%%%%%%%%%%%%%%%%%%%%%%%
%	Initial conditions
rpm_init = 135000; % rpm, constant for model
pp_init = 1.2e5; % plenum pressure at t=0
p01 = p0; % inlet suction pressure is atm.
w_init = 0.12; % comp. mass flow at t=0
wt_init = 0.05;  % throttle mass flow at t=0
pcss_init = 1.95;  % initial comp. char

kt_init = 0.8; % Throttle Control, Throttle opening at start
kt_step_i = 0; % Init. val. step, start for throttle
kt_step_f = 0.6; % Final val. step, reduction in valve opening (%)
t_kt_step = 0.1; % time to init. step

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Delay, disturbance and simulation time
% - sin(t) = p_in_amp*Sin(p_in_f*t+phase) + Bias, bias = p01 cnst.
p_in_amp = 0.001e5; % amplitude of sinusoidal inlet pressure noise
p_in_f = 400; % frequency of sinusoidal inlet pressure noise rad/s

Ts=0.0001; % simulation timestep, change if using fixed step
t_start=0;
t_stop=0.2; %sek, ms... change this to prolong sim
timespan = [t_start,t_stop];
sim_options = simset('solver','ode14x','fixedstep',Ts); %
%sim_options = simset('solver','ode23s'); % ode23s stiff systems
sim('greitzer_mdl',timespan,sim_options);
% 
% %  Solver - [ VariableStepDiscrete |
% %             ode45 | ode23 | ode113 | ode15s | 
% %             ode23s | ode23t | ode23tb |
% %             FixedStepDiscrete |
% %             ode8 | ode5 | ode4 | ode3 | ode2 | ode1 |
% %             ode14x ]
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %           Plot results
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
init_plot; % initialize plot settings
% 
dpnt = 'k-'; % data point symbol and color, subplot2
dfit = dpnt; % 
c = 0.01; % offset plot x-dir
d = -0.05; % offset plot y-dir

%%  Plot, Greitzer mdl
sysplot=figure;
subplot(2,1,1);
plot(t_sim, w_sim.data,t_sim,wt_sim.data,'--');
xlabel('Time [s]');
ylabel('w [kg/s]');
legend('w','w_t');
title(['Throttle step at t=',num2str(t_kt_step),'| k_t from ',...
    num2str(kt_init),' to ', num2str(kt_init-kt_step_f)],'Fontsize',10,'FontWeight','Normal');
grid on
%axis([-inf inf  -inf max(w_g.data)*1.5]);

subplot(2,1,2);
plot(t_sim, pp_sim.data);
xlabel('Time [s]');
ylabel('P_p [Pa]');
grid on
%axis([-inf inf  -inf max(pp_g.data)*1.2]);

% subplot(3,1,3);
% plot(t_sim, p01_sim);
% xlabel('Time [s]');
% ylabel('p_{01} [Pa]');
% grid on
%axis([-inf inf  -inf max(phi_g.data)*1.4]);

compmapplot=figure; % compressor map plot (psi.vs.w.)
plot(x1,y1,dpnt,'linewidth',0.5); text(x1(end)+c,y1(end)+d,'N_{75}');
hold on;
plot(x2,y2,dpnt,'linewidth',0.5); text(x2(end)+c,y2(end)+d,'N_{95}');
plot(x3,y3,dpnt,'linewidth',0.5); text(x3(end)+c,y3(end)+d,'N_{115}');
plot(x4,y4,dpnt,'linewidth',0.5); text(x4(end)+c,y4(end)+d,'N_{135}');
plot(x5,y5,dpnt,'linewidth',0.5); text(x5(end)+c,y5(end)+d,'N_{155}');
plot(x6,y6,dpnt,'linewidth',0.5); text(x6(end)+c,y6(end)+d,'N_{175}');
plot(x7,y7,dpnt,'linewidth',0.5); text(x7(end)+c,y7(end)+d, 'N_{195}');
plot(x10,y10,'k-','linewidth',0.6);
plot(x9,y9,'k--','linewidth',0.6);
plot(map815(91:120,1),map815(91:120,2),'k:','LineWidth',0.5); %70ef
plot(map815(211:240,1),map815(211:240,2),'k:','LineWidth',0.5); %78ef
% -- end plot map data

plot(w_sim.data,dpc_sim.data,'Linewidth',1); % actual resulting data from sim
plot(w_sim.data(1),dpc_sim.data(1),'k*'); % plot starting point for comp.
plot(w_sim.data(end),dpc_sim.data(end),'kx');
hold on;
%axis([-0.05 0.3  0.8  4])
title(['Compressor map, Greitzer mdl. at rpm = ',num2str(rpm_init)...
    ],'Fontsize',10,'FontWeight','Normal');
xlabel 'w [kg/s]'
ylabel '\Delta{p_c} [-]'
grid on
% 
% %%%%%%%%%%%%%%%%%%%%%
% % Print to file
% %%%%%%%%%%%%%%%%%%%%
% %print(sysplot,'fig1_gsys_tr-step','-depsc2','-r300');
% %print(compmapplot,'fig2_gmap_tr-step','-depsc2','-r300');